/**************************************************************************
**       Title: svmtl_dist_float
** Description: svmtl command line application for distance data or 
**              kernel matrix given in single binary file/multiple binary 
**              files in some directory and labels given in a GID-file, i.e.
**              a file with lines as <label> <global_id> 
**              Performing training/testing and model selection 
**              for multiclass data. Sample data is given in the
**              subdirectory data.
**
** the following 3 commands should give the same cross-val results:
**              
** ./svmtl crossval -v 270 -kf rbf -g 0.1 data/heart_scale.vec
**              
** ./svmtl_dist_float crossval -v 270 -kf DS_rbf_file_binary   
**          -kmf data/heart_scale_EDsqr_bin-rows-0-269-cols-0-269.kmf  
**          -g 0.1 data/heart_scale.gid
**
** ./svmtl_dist_float crossval -v 270 -kf DS_rbf_file_multbinary 
**          -kmd data/heart_scale_EDsqr_multbin  
**          -g 0.1 data/heart_scale.gid
** 
** similarly, the following pairs should give the same output and 
** classification results in test_dist_float.output and test.output
**
**  ./svmtl_dist_float train --cost 10 -kf DS_rbf_file_binary 
**          -kmf data/heart_scale_EDsqr_bin-rows-0-269-cols-0-269.kmf  
**          --gamma 0.01 -m test_float_dist.model data/heart_scale_train.gid
**
** ./svmtl_dist_float classify -m test_float_dist.model 
**          -o test_dist_float.output data/heart_scale_test.gid
**
** ./svmtl train --cost 10 -kf rbf --gamma 0.01 
**          -m test.model data/heart_scale_train.vec
**
** ./svmtl classify -m test.model -o test.output data/heart_scale_test.vec
**
**
** Bernard Haasdonk 6.6.2005
**************************************************************************/

#include "SVMApplication.hh" 
#include "GlobalIDFV.hh"  
#include "Kernel_DS_TRIA.hh"  
#include "Kernel_DS_RBF.hh"  
#include "Kernel_FILE.hh"  
#include "KFile_BINARY.hh"  
#include "KFile_MULTBINARY.hh"  
//#include "Kernel_DS_LINEAR.hh"
//#include "Kernel_DS_POLY.hh"  

#include <AlgorithmLists.hh>
//#include <DefaultMultiClassList.hh>
#include "MultiClassSVMOneVsOne.hh"
#include <DefaultTwoClassList.hh>
#include <DefaultOneClassList.hh>
//#include <DefaultKernelList.hh>
#include "LoadSaveASCII.hh"

typedef TLIST_6( 
    svt::Kernel_DS_TRIA< svt::Kernel_FILE< svt::KFile_BINARY> >, 
    svt::Kernel_DS_RBF< svt::Kernel_FILE< svt::KFile_BINARY> >, 
    svt::Kernel_DS_TRIA< svt::Kernel_FILE< svt::KFile_MULTBINARY> >, 
    svt::Kernel_DS_RBF< svt::Kernel_FILE< svt::KFile_MULTBINARY> >, 
    //svt::Kernel_DS_LINEAR, 
    //svt::Kernel_DS_POLY, 
    svt::Kernel_FILE< svt::KFile_BINARY >,
    svt::Kernel_FILE< svt::KFile_MULTBINARY >
    )
    MyKernelList;


typedef TTLIST_1( 
    svt::MultiClassSVMOneVsOne 
    )
    MyMultiClassList;

typedef svt::SVMApplication< svt::GlobalIDFV,
                             svt::AlgorithmLists< MyMultiClassList, 
                                                  svt::DefaultTwoClassList,
                                                  svt::DefaultOneClassList,
                                                  MyKernelList>,
                             svt::LoadSaveASCII>  SVMApplicationDist;

int main( int argc, const char** argv)
{
  SVMApplicationDist sa;
  return sa.main( argc, argv, std::cout);
}

